unit UMain;

{ MeltSim/Compile, Version 1.0 }
{ (C) Copyright, 1998, J.W. Bizzaro }

{ Min stack $00100000 }
{ Max stack $01E84800 }
{ Image base $00400000 }

interface

uses
  Windows, SysUtils, WinTypes, WinProcs, Messages, Classes, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, Buttons, ComCtrls, FMXUtils,
  OleCtrls, Gauges, Menus;

const
  ProgName = 'MeltSim/Compile 1.0';

type
  TFMain = class(TForm)
    ODGenBank: TOpenDialog;
    MainMenu: TMainMenu;
    MIFileMenu: TMenuItem;
    MICompileMenu: TMenuItem;
    MIMeltSimMenu: TMenuItem;
    MIHelpMenu: TMenuItem;
    MIOpenGenBank: TMenuItem;
    MICloseGenBank: TMenuItem;
    MIFileNames: TMenuItem;
    MIExitCompile: TMenuItem;
    MIBeginCompile: TMenuItem;
    MIEndCompile: TMenuItem;
    MIGraph: TMenuItem;
    MIHelpTopics: TMenuItem;
    N2: TMenuItem;
    N1: TMenuItem;
    Divider1: TBevel;
    StatusBar: TStatusBar;
    ProgressBar: TProgressBar;
    PResize: TPanel;
    IUpDown: TImage;
    MIMelt: TMenuItem;
    Memo: TMemo;
    GBSearchCriteria: TGroupBox;
    CBLocator: TComboBox;
    LLocator: TLabel;
    CBPartialSeqs: TCheckBox;
    CBMultiGenes: TCheckBox;
    LLog: TLabel;
    CBChloro: TCheckBox;
    CBMitochon: TCheckBox;
    CBNucleus: TCheckBox;
    Bevel1: TBevel;
    GBOutput: TGroupBox;
    LTitle: TLabel;
    ETitle: TEdit;
    CBMarkEnds: TCheckBox;
    MICompile: TMenuItem;
    procedure MIBeginCompileClick(Sender: TObject);
    procedure MIHelpTopicsClick(Sender: TObject);
    procedure MIOpenGenBankClick(Sender: TObject);
    procedure SetFileNames(Sender: TObject);
    procedure CloseForm(Sender: TObject; var Action: TCloseAction);
    procedure MIExitCompileClick(Sender: TObject);
    procedure CloseFormQuery(Sender: TObject; var CanClose: Boolean);
    procedure MIFileNamesClick(Sender: TObject);
    procedure MIGraphClick(Sender: TObject);
    procedure ResizeForm(Sender: TObject);
    procedure MICloseGenBankClick(Sender: TObject);
    procedure CBLocatorChange(Sender: TObject);
    procedure MIMeltClick(Sender: TObject);
    procedure ShowForm(Sender: TObject);
  private
    { Private declarations }
    procedure ThreadDone(Sender: TObject);
  public
    { Public declarations }
  end;

var
  FMain : TFMain;

implementation

uses
  UCompile, UFiles;

{$R *.DFM}

(******************************************************************************)
{ TFMain }

procedure TFMain.ShowForm(Sender: TObject);
begin

{
ParamCount
ParamStr(1)
}

  if ParamCount > 0
    then
    begin
      FFiles.EGenBankFile.Text := ParamStr(1);
      SetFileNames(Sender);
    end;

end;


(******************************************************************************)


procedure TFMain.MIOpenGenBankClick(Sender: TObject);
{ File is not actually opened, but the file name is determined }

begin

  if ODGenBank.Execute then
    FFiles.EGenBankFile.Text := ODGenBank.FileName;

  { Set file names }
  SetFileNames(Sender);

end;


(******************************************************************************)


procedure TFMain.SetFileNames(Sender: TObject);
begin

  { File names were once set but have been changed - ask to reset Seq and Log }
  if ((FFiles.ESeqFile.Text <> '.seq') or (FFiles.ELogFile.Text <> '.log'))
   and (FFiles.EGenBankFile.Text <> '.gbk')
    then
    begin
      MessageBeep(MB_ICONQUESTION);

      if MessageDlg('Change output file names?', mtConfirmation,
       [mbYes, mbNo], 0) = mrYes
        then
        begin
          FFiles.ESeqFile.Text := ChangeFileExt(FFiles.EGenBankFile.Text, '.seq');
          FFiles.ELogFile.Text := ChangeFileExt(FFiles.EGenBankFile.Text, '.log');
        end;
    end;

  { GenBank/Input file name is set - set Seq and Log }
  if (FFiles.ESeqFile.Text = '.seq') and (FFiles.ELogFile.Text = '.log')
   and (FFiles.EGenBankFile.Text <> '.gbk')
    then
    begin
        FFiles.ESeqFile.Text := ChangeFileExt(FFiles.EGenBankFile.Text, '.seq');
        FFiles.ELogFile.Text := ChangeFileExt(FFiles.EGenBankFile.Text, '.log');
    end;

  { Set title and FMain.Caption }
  if CBLocator.Text <> ''
    then
      ETitle.Text := ExtractFileName(FFiles.EGenBankFile.Text) + ': ' + CBLocator.Text
    else
      ETitle.Text := ExtractFileName(FFiles.EGenBankFile.Text);

  Caption := ProgName + ' - ' + ExtractFileName(FFiles.EGenBankFile.Text);

end;


(******************************************************************************)


procedure TFMain.MICloseGenBankClick(Sender: TObject);
begin

  Memo.Clear;
  FFiles.EGenBankFile.Text := '.gbk';
  FFiles.ESeqFile.Text := '.seq';
  FFiles.ELogFile.Text := '.log';

end;


(******************************************************************************)


procedure TFMain.MIFileNamesClick(Sender: TObject);
begin

  FFiles.Visible := True;

end;


(******************************************************************************)


procedure TFMain.MIBeginCompileClick(Sender: TObject);

var
  I           : Integer;
  MsgReturned : String;

  (**************************************************************************)


  function InputDeterminedValid: Boolean;

  begin

    MsgReturned := '';

    { Check input - list most important here last }
    if CBLocator.Text = ''
      then MsgReturned := 'You must enter a "locator."';

    if not FileExists(FFiles.EGenBankFile.Text)
      then MsgReturned := 'You must open a GenBank file.';

    if MsgReturned = ''
      then
        InputDeterminedValid := True
      else
        InputDeterminedValid := False;

  end;


  (**************************************************************************)


begin { TFMain.MIBeginCompileClick }

  if InputDeterminedValid
    then
    begin
      { Input determined to be valid }
      with TCompileThread.Create(False) do
        OnTerminate := ThreadDone;

      { Reset StatusBar and disable some controls}
      StatusBar.Panels[0].Text := 'Working...';
      ProgressBar.Position := 0;
      MIOpenGenBank.Enabled := False;
      MIBeginCompile.Enabled := False;
    end
    else
    begin
      { Input determined to be invalid }
      MessageBeep(MB_ICONEXCLAMATION);
      MessageDlg(MsgReturned, mtWarning, [mbOK], 0);
    end;

end; { TFMain.MIBeginCompileClick }


(******************************************************************************)


procedure TFMain.ThreadDone(Sender: TObject);
{ code performed at end of main routine }
begin

  { Reset StatusBar and enable some controls}
  StatusBar.Panels[0].Text := 'Ready.';
  ProgressBar.Position := 0;
  MIOpenGenBank.Enabled := True;
  MIBeginCompile.Enabled := True;

end;


(******************************************************************************)


procedure TFMain.MIMeltClick(Sender: TObject);
begin

{ Put something here }

end;


(******************************************************************************)


procedure TFMain.MIGraphClick(Sender: TObject);
begin

{ Put something here }

end;


(******************************************************************************)


procedure TFMain.MIHelpTopicsClick(Sender: TObject);

begin { TFMain.HelpTopicsClick }

  Application.HelpFile := 'LOGHELP.HLP';
  Application.HelpCommand(HELP_FINDER, 0);

{ ExecuteFile('loghelp.hlp', '', '', SW_SHOW); }

end; { TFMain.HelpTopicsClick }


(******************************************************************************)


procedure TFMain.CBLocatorChange(Sender: TObject);
begin

  { Give the output file a title }
  if CBLocator.Text <> ''
    then
      ETitle.Text := ExtractFileName(FFiles.EGenBankFile.Text) + ': ' + CBLocator.Text
    else
      ETitle.Text := ExtractFileName(FFiles.EGenBankFile.Text);

end;


(******************************************************************************)


procedure TFMain.ResizeForm(Sender: TObject);
begin

  if FMain.Height = 473
    then
    begin
      FMain.Height := 67;
      ProgressBar.Top := 8;
      PResize.Top := 7;
    end
    else
    begin
      FMain.Height := 473;
      ProgressBar.Top := 414;
      PResize.Top := 413;
    end;

end;


(******************************************************************************)


procedure TFMain.CloseForm(Sender: TObject; var Action: TCloseAction);
begin
  MessageBeep(MB_ICONQUESTION);

  if MessageDlg('Are you sure you want to exit Compile?', mtConfirmation,
    [mbYes, mbNo], 0) = mrYes
    then
    begin
      Application.Terminate;
    end
    else
      Action := caNone;
end;


(******************************************************************************)


procedure TFMain.CloseFormQuery(Sender: TObject; var CanClose: Boolean);

var
  Action : TCloseAction;

begin
  CloseForm(Sender, Action);

  if Action = caNone
    then
      CanClose := False;
end;


(******************************************************************************)


procedure TFMain.MIExitCompileClick(Sender: TObject);

var
  Action : TCloseAction;

begin

  CloseForm(Sender, Action);

end;


(******************************************************************************)


end.
