unit UMain;

{ MeltSim/Graph, Version 1.0 }
{ (C) Copyright, 1998, J.W. Bizzaro }

interface

uses
  Windows, SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, Buttons, ComCtrls, FMXUtils,
  OleCtrls, Gauges, Menus;

const
  ProgName              = 'MeltSim/Graph 1.0';
  MaxMltDataPoints      = 1001;
  MaxRefDataPoints      = 1001;
  MaxMapDataPoints      = 60060;

type
  MltDataPointType      = Record
                            Temp     : Real;
                            DthetaDT : Double;
                          end; { Record }
  MltDataPointArrayType = Array[1..MaxMltDataPoints] of MltDataPointType;

  RefDataPointType      = Record
                            Temp     : Real;
                            DthetaDT : Double;
                          end; { Record }
  RefDataPointArrayType = Array[1..MaxRefDataPoints] of RefDataPointType;

  MapDataPointType      = Record
                            Temp,
                            Pos   : Real;
                            Theta : Double;
                          end; { Record }
  MapDataPointArrayType = Array[1..MaxMapDataPoints] of MapDataPointType;

  TFMain = class(TForm)
    StatusBar1: TStatusBar;
    PrintDialog1: TPrintDialog;
    MainMenu: TMainMenu;
    MIFileMenu: TMenuItem;
    MIHelpMenu: TMenuItem;
    MIOpenMenu: TMenuItem;
    MIHelpTopics: TMenuItem;
    MIPrintGraph: TMenuItem;
    MIExit: TMenuItem;
    MICloseMenu: TMenuItem;
    MIAutoPrint: TMenuItem;
    Divider1: TBevel;
    MIToolsMenu: TMenuItem;
    MIOpenMeltingCurve: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    MIOpenMeltingMap: TMenuItem;
    MIOpenReferenceCurve: TMenuItem;
    MIViewMeltingMap: TMenuItem;
    MIViewReferenceCurve: TMenuItem;
    MIAdjustAxes: TMenuItem;
    MIFilesNames: TMenuItem;
    N6: TMenuItem;
    MIPrintOptions: TMenuItem;
    MICloseMeltingCurve: TMenuItem;
    MICloseMeltingMap: TMenuItem;
    MICloseReferenceCurve: TMenuItem;
    ProgressBar1: TProgressBar;
    Panel1: TPanel;
    PaintBox1: TPaintBox;
    PResize: TPanel;
    IUpDown: TImage;
    ODMeltingCurve: TOpenDialog;
    ODMeltingMap: TOpenDialog;
    MICalculateTms: TMenuItem;
    ODReferenceCurve: TOpenDialog;
    MIViewMenu: TMenuItem;
    MIViewMeltingCurve: TMenuItem;
    N3: TMenuItem;
    MIViewMeltingCurveTm: TMenuItem;
    MIViewReferenceCurveTm: TMenuItem;
    Timer: TTimer;
    MIMeltSimMenu: TMenuItem;
    MICompile: TMenuItem;
    MILaunchMelt: TMenuItem;
    MIGraph: TMenuItem;
    procedure MIHelpTopicsClick(Sender: TObject);
    procedure OnPaintPB1(Sender: TObject);
    procedure AutoScale;
    procedure ClearMlt(MltDataPoint : MltDataPointArrayType);
    procedure MIPrintGraphClick(Sender: TObject);
    procedure MIAutoPrintClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure MIExitClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure MIAdjustAxesClick(Sender: TObject);
    procedure MIFilesNamesClick(Sender: TObject);
    procedure MIPrintOptionsClick(Sender: TObject);
    procedure MIViewMeltingCurveClick(Sender: TObject);
    procedure MIViewMeltingMapClick(Sender: TObject);
    procedure MIViewReferenceCurveClick(Sender: TObject);
    procedure Resize(Sender: TObject);
    procedure OpenMeltingCurve(Sender: TObject);
    procedure OpenMeltingMap(Sender: TObject);
    procedure OpenReferenceCurve(Sender: TObject);
    procedure MICalculateTmsClick(Sender: TObject);
    procedure MIViewMeltingCurveTmClick(Sender: TObject);
    procedure MIViewReferenceCurveTmClick(Sender: TObject);
    procedure ShowForm(Sender: TObject);
  private
    { Private declarations }
    procedure ThreadDone(Sender: TObject);
  public
    { Public declarations }
  end;

var
  Code,
  VertSpacing,
  DomainsPerBlock       : Integer;
  DOMAIN,
  MltDataPointNumber,
  RefDataPointNumber,
  LastMltDataPoint,
  LastRefDataPoint,
  MapDataPointNumber    : LongInt;
  TotalBasesMelted,
  MltHighValue,
  MltMax,
  MltTm,
  RefTm                 : Real;
  BlockMltActive,
  ClearPlots            : Boolean;
  OutFile1Name,
  OutFile2Name,
  OutFile3Name,
  LogFileName,
  TitleLine1,
  TitleLine2,
  TitleLine3,
  TempString,
  TotalBasesMeltedString,
  MltMaxString,
  MltTmString,
  RefTmString           : String;
  TotalMltDataPoint,                             { Array for total melting curve }
  BlockMltDataPoint     : MltDataPointArrayType; { Array for block melting curve }
  TotalRefDataPoint     : RefDataPointArrayType; { Array for reference curve }
  MapDataPoint          : MapDataPointArrayType; { Array for positional plot }
  InFile,
  OutFile1,
  OutFile2,
  OutFile3,
  LogFile               : TextFile;
  FMain                 : TFMain;

procedure ReadNextString(const InFile : TextFile; var NextString : String);

implementation

uses
  URepaint, UPrint, UPrintOpts, UAxes, UFiles;

{$R *.DFM}

(******************************************************************************)
{ TFMain }

procedure TFMain.ShowForm(Sender: TObject);

begin

  { Set file names to launch parameters }
  if ParamCount > 0 then
  begin
    FFiles.EMltFile.Text := ParamStr(1);
    OpenMeltingCurve(Sender);
  end;

  if ParamCount > 1 then
  begin
    FFiles.ERefFile.Text := ParamStr(2);
    OpenReferenceCurve(Sender);
  end;

  if ParamCount > 2 then
  begin
    FFiles.EMapFile.Text := ParamStr(3);
    OpenMeltingMap(Sender);
  end;

end;


(******************************************************************************)


procedure ReadNextString(const InFile : TextFile; var NextString : String);

var
  Character : Char;

begin  { ReadNextString }

  NextString := '';
  Read(InFile, Character);

  { skip over non-characters }
  while (not EOF(InFile)) and (not (Character in ['0'..'9', '.', '+', '-', 'E'])) do
    Read(InFile, Character);

  { read in NextString while there are characters }
  while (not EOF(InFile)) and (Character in ['0'..'9', '.', '+', '-', 'E']) do
  begin
    NextString := NextString + Character;
    Read(InFile, Character);
  end;

  if NextString = ''
    then
      NextString := '0';

end; { ReadNextString }


(******************************************************************************)


procedure TFMain.OpenMeltingCurve(Sender: TObject);

var
  FirstChar   : Char;
  NextString  : String;
  MltInFile   : TextFile;

begin
  if Sender = MIOpenMeltingCurve then
    if ODMeltingCurve.Execute then
      FFiles.EMltFile.Text := ODMeltingCurve.FileName;

  { Assign and open MltInFile }
  AssignFile(MltInFile, FFiles.EMltFile.Text);
  Reset(MltInFile);

  { Extract title from MltInFile }
  Read(MltInFile, FirstChar);
  if FirstChar = '#'
    then
      Readln(MltInFile, TitleLine1) { Read title - rest of line }
    else
    begin
      { Go back to start }
      CloseFile(MltInFile);
      AssignFile(MltInFile, FFiles.EMltFile.Text);
      Reset(MltInFile);
      { Default title is InFile name }
      TitleLine1 := ExtractFileName(FFiles.EMltFile.Text);
    end;

  { Set FMain.Caption }
  FMain.Caption := ProgName + ' - ' + ExtractFileName(FFiles.EMltFile.Text);

  { Read coordinates into array }
  LastMltDataPoint := 1;

  While not EOF(MltInFile) do
    begin
      ReadNextString(MltInFile, NextString);
      TotalMltDataPoint[LastMltDataPoint].Temp := StrToFloat(NextString);
      ReadNextString(MltInFile, NextString);
      TotalMltDataPoint[LastMltDataPoint].DthetaDT := StrToFloat(NextString);
      LastMltDataPoint := LastMltDataPoint + 1;
    end;

  { Values needed for graphing }
  { Will read one too many datapoints before EOF is reached }
  LastMltDataPoint := LastMltDataPoint - 1;
  MltDataPointNumber := LastMltDataPoint;

  { Close MltInFile }
  CloseFile(MltInFile);

  { Check the menu item that shows melting curve }
  MIViewMeltingCurve.Checked := True;

  { Repaint }
  OnPaintPB1(Sender);
end;


(******************************************************************************)


procedure TFMain.OpenReferenceCurve(Sender: TObject);

var
  FirstChar   : Char;
  NextString  : String;
  RefInFile   : TextFile;

begin
  if Sender = MIOpenReferenceCurve then
    if ODMeltingCurve.Execute then
      FFiles.ERefFile.Text := ODMeltingCurve.FileName;

  { Assign and open RefInFile }
  AssignFile(RefInFile, FFiles.ERefFile.Text);
  Reset(RefInFile);

  { Extract title from RefInFile }
  Read(RefInFile, FirstChar);
  if FirstChar = '#'
    then
      Readln(RefInFile, TitleLine2) { Read title - rest of line }
    else
    begin
      { Go back to start }
      CloseFile(RefInFile);
      AssignFile(RefInFile, FFiles.ERefFile.Text);
      Reset(RefInFile);
      { Default title is InFile name }
      TitleLine2 := ExtractFileName(FFiles.ERefFile.Text);
    end;

  { Set FMain.Caption }
  FMain.Caption := ProgName + ' - ' + ExtractFileName(FFiles.ERefFile.Text);

  { Read coordinates into array }
  LastRefDataPoint := 1;

  While not EOF(RefInFile) do
    begin
      ReadNextString(RefInFile, NextString);
      TotalRefDataPoint[LastRefDataPoint].Temp := StrToFloat(NextString);
      ReadNextString(RefInFile, NextString);
      TotalRefDataPoint[LastRefDataPoint].DthetaDT := StrToFloat(NextString);
      LastRefDataPoint := LastRefDataPoint + 1;
    end;

  { Values needed for graphing }
  { Will read one too many datapoints before EOF is reached }
  LastRefDataPoint := LastRefDataPoint - 1;
  RefDataPointNumber := LastRefDataPoint;

  { Close RefInFile }
  CloseFile(RefInFile);

  { Check the menu item that shows reference curve }
  MIViewReferenceCurve.Checked := True;

  { Repaint }
  OnPaintPB1(Sender);
end;


(******************************************************************************)


procedure TFMain.OpenMeltingMap(Sender: TObject);

var
  FirstChar   : Char;
  NextString  : String;
  MapInFile   : TextFile;

begin
  if Sender = MIOpenMeltingMap then
    if ODMeltingMap.Execute then
      FFiles.EMapFile.Text := ODMeltingMap.FileName;

  { Assign and open MapInFile }
  AssignFile(MapInFile, FFiles.EMapFile.Text);
  Reset(MapInFile);

  { Extract title from MapInFile }
  Read(MapInFile, FirstChar);
  if FirstChar = '#'
    then
      Readln(MapInFile, TitleLine2) { Read title - rest of line }
    else
    begin
      { Go back to start }
      CloseFile(MapInFile);
      AssignFile(MapInFile, FFiles.EMapFile.Text);
      Reset(MapInFile);
      { Default title is MapInFile name }
      TitleLine3 := ExtractFileName(FFiles.EMapFile.Text);
    end;

  { Set FMain.Caption }
  FMain.Caption := ProgName + ' - ' + ExtractFileName(FFiles.EMapFile.Text);

  { Read coordinates into array }
  MapDataPointNumber := 1;
  DOMAIN := 0;
  DomainsPerBlock := 0;


  While not EOF(MapInFile) do
    begin
      { Read Temp value }
      ReadNextString(MapInFile, NextString);
      MapDataPoint[MapDataPointNumber].Temp := StrToFloat(NextString);

      { Read Pos value }
      ReadNextString(MapInFile, NextString);
      MapDataPoint[MapDataPointNumber].Pos := StrToInt(NextString);

      { Find the DOMAIN size from Pos }
      if MapDataPointNumber = 3 { DOMAIN might vary - average between 2 sets }
        then
          DOMAIN := Round(((MapDataPoint[3].Pos - MapDataPoint[2].Pos)
           + (MapDataPoint[2].Pos - MapDataPoint[1].Pos)) / 2);

      { Find DomainsPerBlock from Pos }
      { When Pos repeats, we have read in one block }
      if (MapDataPoint[MapDataPointNumber].Pos = MapDataPoint[1].Pos)
       and (DomainsPerBlock = 0) { Make sure it's the first repeat }
        then
          DomainsPerBlock := MapDataPointNumber - 1;

      { Read Theta value }
      ReadNextString(MapInFile, NextString);
      MapDataPoint[MapDataPointNumber].Theta := StrToFloat(NextString);

      MapDataPointNumber := MapDataPointNumber + 1;
    end;

  { Other value needed for graphing }
  { Will read 2 too many datapoints before EOF is reached }
  MapDataPointNumber := MapDataPointNumber - 2;

  { Close MapInFile }
  CloseFile(MapInFile);

  { Check the menu item that shows melting map }
  MIViewMeltingMap.Checked := True;

  { Repaint }
  OnPaintPB1(Sender);
end;


(******************************************************************************)


procedure TFMain.MIPrintGraphClick(Sender: TObject);

var
  X : Integer;

begin { TFMain.Print1Click }

  if PrintDialog1.Execute
    then
      for X := 1 to PrintDialog1.Copies do
        with TPrintThread.Create(False) do
         OnTerminate := ThreadDone;

end; { TFMain.Print1Click }


(******************************************************************************)


procedure TFMain.MIAutoPrintClick(Sender: TObject);
begin

  if not MIAutoPrint.Checked
    then
      MIAutoPrint.Checked := True
    else
      MIAutoPrint.Checked := False;

end;


(******************************************************************************)


procedure TFMain.MIHelpTopicsClick(Sender: TObject);
{ show read me file }

begin { TFMain.Manual1Click }

  ExecuteFile('meltsim.txt', '', '', SW_SHOW);

end; { TFMain.Manual1Click }


(******************************************************************************)


procedure TFMain.OnPaintPB1(Sender: TObject);
{ Repaint important graphics in PaintBox1 }

begin

  RepaintPB1(Sender);

end;


(******************************************************************************)


procedure TFMain.ClearMlt(MltDataPoint : MltDataPointArrayType);

var
  X : Integer;

begin { TFMain.ClearMlt }

  { erase melting curves in PaintBox1 }
  if MIViewMeltingCurve.Checked
    then
    begin
      PaintBox1.Canvas.Pen.Color := clWhite;
      X := 1;
      PaintBox1.Canvas.MoveTo(Round((MltDataPoint[X].Temp - 60) * 7 + 60),
       (340 - Round(MltDataPoint[X].DthetaDT * VertSpacing)));

      for X := 2 to LastMltDataPoint do
        if (340 - Round(MltDataPoint[X].DthetaDT * VertSpacing)) > 60
          then
            PaintBox1.Canvas.LineTo(Round((MltDataPoint[X].Temp - 60) * 7 + 60),
             (340 - Round(MltDataPoint[X].DthetaDT * VertSpacing)))
          else
            PaintBox1.Canvas.LineTo(Round((MltDataPoint[X].Temp - 60) * 7 + 60),
             60);
    end;

end; { TFMain.ClearMlt }


(******************************************************************************)


procedure TFMain.AutoScale;

var
  Sender : TObject;

begin { TFMain.AutoScale }

  ClearMlt(TotalMltDataPoint);

  FAxes.UDMltMax.Position := Round(100 / (MltHighValue * 2));
  while (FAxes.UDMltMax.Position MOD 10) <> 0 do
    FAxes.UDMltMax.Position := FAxes.UDMltMax.Position - 1;

  { change value in edit box }
  VertSpacing := Round(560 * FAxes.UDMltMax.Position / 100); { default scale = 0.5 }
  MltMax := 560 / VertSpacing * 0.5; { for y-axis max value }
  Str(MltMax:0:4, MltMaxString);
  FAxes.EMltMax.Text := MltMaxString;

  { Repaint }
  ClearPlots := True;
  OnPaintPB1(Sender);

end; { TFMain.AutoScale }


(******************************************************************************)


procedure TFMain.ThreadDone(Sender: TObject);
{ code performed at end of main routine }
begin

  MIOpenMenu.Enabled := True;
  MIPrintGraph.Enabled := True;

end;


(******************************************************************************)


procedure TFMain.MIAdjustAxesClick(Sender: TObject);
begin

  FAxes.Visible := True;

end;


(******************************************************************************)


procedure TFMain.MIExitClick(Sender: TObject);

var
  Action : TCloseAction;

begin

  FormClose(Sender, Action);

end;


(******************************************************************************)


procedure TFMain.MIFilesNamesClick(Sender: TObject);
begin

  FFiles.Visible := True;

end;


(******************************************************************************)


procedure TFMain.MIPrintOptionsClick(Sender: TObject);
begin

  FPrintOpts.Visible := True;

end;


(******************************************************************************)


procedure TFMain.MIViewMeltingCurveClick(Sender: TObject);
begin

  if not MIViewMeltingCurve.Checked
    then
      MIViewMeltingCurve.Checked := True
    else
    begin
      MIViewMeltingCurve.Checked := False;
      MIViewMeltingCurveTm.Checked := False;
      ClearPlots := True;
    end;

  { Repaint }
  OnPaintPB1(Sender);

end;


(******************************************************************************)


procedure TFMain.MIViewReferenceCurveClick(Sender: TObject);
begin

  if not MIViewReferenceCurve.Checked
    then
      MIViewReferenceCurve.Checked := True
    else
    begin
      MIViewReferenceCurve.Checked := False;
      MIViewReferenceCurveTm.Checked := False;
      ClearPlots := True;
    end;

  { Repaint }
  OnPaintPB1(Sender);

end;


(******************************************************************************)


procedure TFMain.MIViewMeltingMapClick(Sender: TObject);
begin

  if not MIViewMeltingMap.Checked
    then
      MIViewMeltingMap.Checked := True
    else
    begin
      MIViewMeltingMap.Checked := False;
      ClearPlots := True;
    end;

  { Repaint }
  OnPaintPB1(Sender);

end;


(******************************************************************************)


procedure TFMain.Resize(Sender: TObject);
begin

  if FMain.Height = 473
    then
    begin
      FMain.Height := 67;
      ProgressBar1.Top := 8;
      PResize.Top := 7;
    end
    else
    begin
      FMain.Height := 473;
      ProgressBar1.Top := 414;
      PResize.Top := 413;
    end;

end;


(******************************************************************************)


procedure TFMain.MICalculateTmsClick(Sender: TObject);

var
  X           : Integer;
  Numerator,
  SumDthetaDT : Real;

begin

  { Calculate melting curve Tm }

  Numerator := 0;
  SumDthetaDT := 0;

  for X := 1 to (MltDataPointNumber - 1) do
  begin
    Numerator := Numerator + TotalMltDataPoint[X].Temp * TotalMltDataPoint[X].DthetaDT;
    SumDthetaDT := SumDthetaDT + TotalMltDataPoint[X].DthetaDT;
  end;

  MltTm := Numerator / SumDthetaDT;

  if MIViewMeltingCurve.Checked
    then
      MIViewMeltingCurveTm.Checked := True;

  { Calculate reference curve Tm }

  Numerator := 0;
  SumDthetaDT := 0;

  for X := 1 to (RefDataPointNumber - 1) do
  begin
    Numerator := Numerator + TotalRefDataPoint[X].Temp * TotalRefDataPoint[X].DthetaDT;
    SumDthetaDT := SumDthetaDT + TotalRefDataPoint[X].DthetaDT;
  end;

  RefTm := Numerator / SumDthetaDT;

  if MIViewReferenceCurve.Checked
    then
      MIViewReferenceCurveTm.Checked := True;

  { Repaint }
  OnPaintPB1(Sender);

end;


(******************************************************************************)


procedure TFMain.MIViewMeltingCurveTmClick(Sender: TObject);
begin

  if not MIViewMeltingCurveTm.Checked
    then
      MIViewMeltingCurveTm.Checked := True
    else
    begin
      MIViewMeltingCurveTm.Checked := False;
      ClearPlots := True;
    end;

  { Repaint }
  OnPaintPB1(Sender);

end;


(******************************************************************************)


procedure TFMain.MIViewReferenceCurveTmClick(Sender: TObject);
begin

  if not MIViewReferenceCurveTm.Checked
    then
      MIViewReferenceCurveTm.Checked := True
    else
    begin
      MIViewReferenceCurveTm.Checked := False;
      ClearPlots := True;
    end;

  { Repaint }
  OnPaintPB1(Sender);

end;


(******************************************************************************)


procedure TFMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin

  MessageBeep(MB_ICONQUESTION);

  if MessageDlg('Are you sure you want to exit Graph?', mtConfirmation,
    [mbYes, mbNo], 0) = mrYes
    then
    begin
      Application.Terminate;
    end
    else
      Action := caNone;

end;


(******************************************************************************)


procedure TFMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);

var
  Action : TCloseAction;

begin

  FormClose(Sender, Action);

  if Action = caNone
    then
      CanClose := False;

end;


(******************************************************************************)


end.
