unit UPrint;

interface

uses
  Classes, Printers, SysUtils;

type
  TPrintThread = class(TThread)
  private
    { Private declarations }
  protected
    procedure Execute; override;
  end;

implementation

uses
  UMain, UPrintOpts, UAxes, UFiles;

{ Important: Methods and properties of objects in VCL can only be used in a
  method called using Synchronize, for example,

      Synchronize(UpdateCaption);

  and UpdateCaption could look like,

    procedure TPrintThread.UpdateCaption;
    begin
      Form1.Caption := 'Updated in a thread';
    end; }


(******************************************************************************)
{ TPrintThread }

procedure TPrintThread.Execute;
{ Place thread code here }

const
  XaxisStart        = 60;
  YaxisStart        = 60;
  XaxisFinal        = 340;
  YaxisFinal        = 340;
  YaxisSize         = 280;
  XaxisSize         = 280;

var
  MapMoveToX,
  MapMoveToY,
  MapLineToX,
  MapLineToY,
  MapStart,
  MapScale,
  X                 : Integer;
  PageFactor,
  TempStart,
  TempFinal,
  TempScale,
  LastTempUsed      : Real;
  MapString         : String;


  (**************************************************************************)


  procedure PrintCommonParts;
  { These parts are present on most graphs }

  var
    TickDegs,
    X         : Integer;

  begin { PrintCommonParts }
    { Draw axes in Printer }
    { Draw axes }
    Printer.Canvas.Pen.Color := clBlack;
    Printer.Canvas.MoveTo(Round(60 * PageFactor), Round(60 * PageFactor));
    Printer.Canvas.LineTo(Round(340 * PageFactor), Round(60 * PageFactor));
    Printer.Canvas.LineTo(Round(340 * PageFactor), Round(340 * PageFactor));
    Printer.Canvas.LineTo(Round(60 * PageFactor), Round(340 * PageFactor));
    Printer.Canvas.LineTo(Round(60 * PageFactor), Round(60 * PageFactor));

    { Draw x-axis, top and bottom }
    { Resolution at 0.01C }
    { Variable ticks }
    case (Round(TempFinal * 100) - Round(TempStart * 100)) of
      0..1000     : TickDegs := 100; { 0 to 10.00C gives 1.00C ticks }
      1001..4000  : TickDegs := 500; { 10.01C to 40.00C gives 5.00C ticks }
      else          TickDegs := 1000; { More than 40.00C gives 10.00C ticks }
    end; { case }
    for X := Round(TempStart * 100) to Round(TempFinal * 100) do
      if (X MOD TickDegs = 0) { Tick every TickDegs / 100 degrees }
       and (Round((X / 100 - TempStart) * TempScale + 60) >= 60)
       and (Round((X / 100 - TempStart) * TempScale + 60) <= 340)
        then
        begin
          { Bottom }
          Printer.Canvas.MoveTo(Round((X / 100 - TempStart) * TempScale * PageFactor + 60 * PageFactor), Round(340 * PageFactor));
          Printer.Canvas.LineTo(Round((X / 100 - TempStart) * TempScale * PageFactor + 60 * PageFactor), Round(342 * PageFactor));
          if X MOD (TickDegs * 2) = 0 { Major Tick every 2 minor }
            then
            begin
              Printer.Canvas.LineTo(Round((X / 100 - TempStart) * TempScale * PageFactor + 60 * PageFactor), Round(345 * PageFactor));
              { Value }
              Printer.Canvas.TextOut(Round((X / 100 - TempStart) * TempScale * PageFactor + (60 - 5) * PageFactor), Round(351 * PageFactor), IntToStr(Round(X / 100)));
            end;
          { Top }
          Printer.Canvas.MoveTo(Round((X / 100 - TempStart) * TempScale * PageFactor + 60 * PageFactor), Round(60 * PageFactor));
          Printer.Canvas.LineTo(Round((X / 100 - TempStart) * TempScale * PageFactor + 60 * PageFactor), Round(58 * PageFactor));
          if X MOD (TickDegs * 2) = 0 { Major Tick every 2 minor }
            then
              Printer.Canvas.LineTo(Round((X / 100 - TempStart) * TempScale * PageFactor + 60 * PageFactor), Round(55 * PageFactor));
        end;
    { Label }
    Printer.Canvas.Font.Style := [fsBold];
    Printer.Canvas.TextOut(Round(60 * PageFactor), Round(10 * PageFactor), TitleLine1);
    Printer.Canvas.TextOut(Round(60 * PageFactor), Round(22 * PageFactor), TitleLine2);
    Printer.Canvas.TextOut(Round(60 * PageFactor), Round(34 * PageFactor), TitleLine3);
    Printer.Canvas.TextOut(Round(158 * PageFactor), Round(370 * PageFactor), 'Temperature (C)');
    Printer.Canvas.Font.Style := [];

    { Draw left y-axis }
    { Ticks }
    Printer.Canvas.MoveTo(Round(60 * PageFactor), Round(60 * PageFactor));
    Printer.Canvas.LineTo(Round(55 * PageFactor), Round(60 * PageFactor));
    Printer.Canvas.MoveTo(Round(60 * PageFactor), Round(200 * PageFactor));
    Printer.Canvas.LineTo(Round(55 * PageFactor), Round(200 * PageFactor));
    Printer.Canvas.MoveTo(Round(60 * PageFactor), Round(340 * PageFactor));
    Printer.Canvas.LineTo(Round(55 * PageFactor), Round(340 * PageFactor));
    { Values }
    Printer.Canvas.TextOut(Round(41 * PageFactor), Round(335 * PageFactor), '0');
    MltMax := 560 / VertSpacing * 0.5; { For y-axis max value }
    Str(MltMax:0:1, MltMaxString);
    Printer.Canvas.TextOut(Round(32 * PageFactor), Round(55 * PageFactor), MltMaxString);
    { Label }
    Printer.Canvas.Font.Style := [fsBold];
    Printer.Canvas.TextOut(Round(8 * PageFactor), Round(194 * PageFactor), 'd');
    Printer.Canvas.Font.Name := 'Symbol';
    Printer.Canvas.TextOut(Round(14.5 * PageFactor), Round(193 * PageFactor), 'q');
    Printer.Canvas.Font.Name := 'Arial';
    Printer.Canvas.TextOut(Round(20.5 * PageFactor), Round(194 * PageFactor), '/dT');
    Printer.Canvas.Font.Style := [];

    { Draw right y-axis ticks...non-positional plot }
    if not(FMain.MIViewMeltingMap.Checked)
      then
      begin
        Printer.Canvas.MoveTo(Round(340 * PageFactor), Round(60 * PageFactor));
        Printer.Canvas.LineTo(Round(345 * PageFactor), Round(60 * PageFactor));
        Printer.Canvas.MoveTo(Round(340 * PageFactor), Round(200 * PageFactor));
        Printer.Canvas.LineTo(Round(345 * PageFactor), Round(200 * PageFactor));
        Printer.Canvas.MoveTo(Round(340 * PageFactor), Round(340 * PageFactor));
        Printer.Canvas.LineTo(Round(345 * PageFactor), Round(340 * PageFactor));
      end;

    { Print legend or log }
    if not FPrintOpts.CBPrintLogAtBottom.Checked
      then
      { Print legend }
      begin
        Printer.Canvas.Font.Style := [fsBold];
        Printer.Canvas.TextOut(Round(60 * PageFactor), Round(400 * PageFactor), FPrintOpts.Memo.Lines[1]);
        Printer.Canvas.TextOut(Round(60 * PageFactor), Round(412 * PageFactor), FPrintOpts.Memo.Lines[2]);
        Printer.Canvas.TextOut(Round(60 * PageFactor), Round(424 * PageFactor), FPrintOpts.Memo.Lines[3]);
        Printer.Canvas.TextOut(Round(60 * PageFactor), Round(436 * PageFactor), FPrintOpts.Memo.Lines[4]);
        Printer.Canvas.Font.Style := [];
      end
      else
      { Print short log }
      begin
        Printer.Canvas.Font.Name := 'Courier';
        Printer.Canvas.Font.Size := 12;
        Printer.Canvas.TextOut(Round(10 * PageFactor), Round(400 * PageFactor), DateToStr(Date) + ' ' + TimeToStr(Time));
        Printer.Canvas.TextOut(Round(10 * PageFactor), Round(412 * PageFactor), 'Melting Curve File: ' + ExtractFileName(FFiles.EMltFile.Text));
        Printer.Canvas.TextOut(Round(10 * PageFactor), Round(424 * PageFactor), 'Reference Curve File: ' + ExtractFileName(FFiles.ERefFile.Text));
        Printer.Canvas.TextOut(Round(10 * PageFactor), Round(436 * PageFactor), 'Melting Map File: ' + ExtractFileName(FFiles.EMapFile.Text));
        Printer.Canvas.Font.Name := 'Arial';
        Printer.Canvas.Font.Size := 15;
      end;
  end; { PrintCommonParts }


  (**************************************************************************)


begin { TFMain.OnPrint }
  Printer.BeginDoc;

  TempStart := StrToFloat(FAxes.ETempMin.Text);
  TempFinal := StrToFloat(FAxes.ETempMax.Text);
  TempScale := (XaxisFinal - XaxisStart) / (TempFinal - TempStart);

  PageFactor := Printer.PageWidth / 400; { to increase graphic to page width }

  Printer.Canvas.Font.Name := 'Arial';
  Printer.Canvas.Font.Size := 15;
  Printer.Canvas.Font.Style := [];
  Printer.Canvas.Pen.Width := Round(1 * PageFactor);
  Printer.Canvas.Pen.Color := clBlack;

  { Print positional plot }
  { Drawn first to avoid erasing other curves }
  if FMain.MIViewMeltingMap.Checked
    then
    begin
      { Determine scale and bottom of positional plot }
      MapScale := Round(YaxisSize * DomainsPerBlock / (StrToInt(FAxes.ELastDomain.Text) - StrToInt(FAxes.EFirstDomain.Text) + 1));
      MapStart := 339 + Round((StrToInt(FAxes.EFirstDomain.Text) - 1) / DomainsPerBlock * MapScale);
      LastTempUsed := MapDataPoint[DomainsPerBlock + 1].Temp;

      for X := (DomainsPerBlock + 1) to MapDataPointNumber do
      begin
         { Determine temperature increment displayed }
        if (MapDataPoint[X].Temp >= LastTempUsed + StrToFloat(FAxes.ETempInc.Text))
         { Must be within boundaries }
         and (Round((MapDataPoint[X].Temp - TempStart) * TempScale + 60) > 60)
         and (Round((MapDataPoint[X].Temp - TempStart) * TempScale + 60) < 340)
         { Determine domains to be displayed }
         and (X >= Trunc((X - 1) / DomainsPerBlock) * DomainsPerBlock + StrToInt(FAxes.EFirstDomain.Text))
         and (X <= Trunc((X - 1) / DomainsPerBlock) * DomainsPerBlock + StrToInt(FAxes.ELastDomain.Text))
          then
          begin
            { Make LastTempUsed the current temp if next pass increases temp }
            if X = Trunc((X - 1) / DomainsPerBlock) * DomainsPerBlock + StrToInt(FAxes.ELastDomain.Text)
              then
                LastTempUsed := MapDataPoint[X].Temp;

            { Print varying shades of gray }
            if FPrintOpts.RBshade.Checked
              then
              begin
                { Brush used because some printers use only black for the pen }
                Printer.Canvas.Pen.Color := clWhite;
                Printer.Canvas.Pen.Width := 0;
                { Determine color to use }
                if MapDataPoint[X].Theta < 0.35
                  then
                    Printer.Canvas.Brush.Color := $00c0c0c0; { Light gray }
                if (MapDataPoint[X].Theta >= 0.35) and (MapDataPoint[X].Theta < 0.45)
                  then
                    Printer.Canvas.Brush.Color := $00606060; { Medium gray }
                if (MapDataPoint[X].Theta >= 0.45) and (MapDataPoint[X].Theta < 0.55)
                  then
                    Printer.Canvas.Brush.Color := $00202020; { Dark gray }
                if (MapDataPoint[X].Theta >= 0.55) and (MapDataPoint[X].Theta < 0.65)
                  then
                    Printer.Canvas.Brush.Color := $00606060; { Medium gray }
                if MapDataPoint[X].Theta >= 0.65
                  then
                    Printer.Canvas.Brush.Color := clWhite;

                { Draw positional melt on printer using brush }
                Printer.Canvas.Rectangle(Round((MapDataPoint[X].Temp - TempStart) * TempScale * PageFactor + 60 * PageFactor),
                 Round((MapStart * PageFactor) - Round((MapDataPoint[X].Pos + DOMAIN / 2) / (DOMAIN * DomainsPerBlock) * (MapScale - 1) * PageFactor) + 1 * PageFactor),
                 Round((MapDataPoint[X].Temp - TempStart) * TempScale * PageFactor) + Round(60 * PageFactor) + Round(1 * PageFactor),
                 Round((MapStart * PageFactor) - Round((MapDataPoint[X].Pos - DOMAIN / 2) / (DOMAIN * DomainsPerBlock) * (MapScale - 1) * PageFactor) + 1 * PageFactor));
              end;

            { Print varying line thicknesses }
            if FPrintOpts.RBthickness.Checked
              then
              begin
                Printer.Canvas.Pen.Color := clBlack;
                { Determine line thickness to use }
                if MapDataPoint[X].Theta < 0.35
                  then
                    Printer.Canvas.Pen.Width := Round(0.25 * PageFactor);
                if (MapDataPoint[X].Theta >= 0.35) and (MapDataPoint[X].Theta < 0.45)
                  then
                    Printer.Canvas.Pen.Width := Round(0.50 * PageFactor);
                if (MapDataPoint[X].Theta >= 0.45) and (MapDataPoint[X].Theta < 0.55)
                  then
                    Printer.Canvas.Pen.Width := Round(1 * PageFactor);
                if (MapDataPoint[X].Theta >= 0.55) and (MapDataPoint[X].Theta < 0.65)
                  then
                    Printer.Canvas.Pen.Width := Round(0.50 * PageFactor);
                if MapDataPoint[X].Theta >= 0.65
                  then
                    Printer.Canvas.Pen.Color := clNone;

                { Draw positional melt on printer using pen }
                MapMoveToX := Round((MapDataPoint[X].Temp - TempStart) * TempScale * PageFactor + 60 * PageFactor);
                MapMoveToY := Round(MapStart * PageFactor - Round((MapDataPoint[X].Pos - DOMAIN / 2) / (DOMAIN * DomainsPerBlock) * (MapScale - 1) * PageFactor));
                MapLineToX := MapMoveToX;
                MapLineToY := Round(MapStart * PageFactor - Round((MapDataPoint[X].Pos + DOMAIN / 2) / (DOMAIN * DomainsPerBlock) * (MapScale - 1) * PageFactor) + 1 * PageFactor);

                Printer.Canvas.MoveTo(MapMoveToX, MapMoveToY);
                Printer.Canvas.LineTo(MapLineToX, MapLineToY);
              end;
          end; { if }
      end; { for }

      Printer.Canvas.Pen.Width := Round(1 * PageFactor);
      Printer.Canvas.Pen.Color := clBlack;
      Printer.Canvas.Brush.Color := clNone;

      PrintCommonParts;

      { Remove left y-axis ticks, etc....may be redrawn later }
      Printer.Canvas.Brush.Color := clWhite;
      Printer.Canvas.FillRect(Rect(0, Round(50 * PageFactor), Round(59 * PageFactor), Round(350 * PageFactor)));
      { Remove right y-axis ticks...to be replaced below }
      Printer.Canvas.FillRect(Rect(Round(341 * PageFactor), Round(50 * PageFactor), Round(400 * PageFactor), Round(350 * PageFactor)));

      { Draw right y-axis ticks, etc. }
      for X := StrToInt(FAxes.EFirstDomain.Text) to StrToInt(FAxes.ELastDomain.Text) do
      begin
        if X MOD 3 = 0 { Major tick every 3 domains }
          then
          begin
            Printer.Canvas.MoveTo(Round(340 * PageFactor), Round(MapStart * PageFactor + Round(1 / DomainsPerBlock / 2 * MapScale * PageFactor) + 1 * PageFactor - Round(X / DomainsPerBlock * MapScale * PageFactor)));
            Printer.Canvas.LineTo(Round(345 * PageFactor), Round(MapStart * PageFactor + Round(1 / DomainsPerBlock / 2 * MapScale * PageFactor) + 1 * PageFactor - Round(X / DomainsPerBlock * MapScale * PageFactor)));
            Str(((MapDataPoint[X].Pos + StrToInt(FAxes.ETicksLabeledFrom.Text) - 1) / 1000):0:StrToInt(FAxes.ELabelDecimalPlaces.Text), MapString);
            Printer.Canvas.TextOut(Round(355 * PageFactor), Round(MapStart * PageFactor + Round(1 / DomainsPerBlock / 2 * MapScale * PageFactor) + 1 * PageFactor - Round(X / DomainsPerBlock * MapScale * PageFactor) - 6 * PageFactor), MapString);
          end
          else { Minor ticks }
          begin
            Printer.Canvas.MoveTo(Round(340 * PageFactor), Round(MapStart * PageFactor + Round(1 / DomainsPerBlock / 2 * MapScale * PageFactor) + 1 * PageFactor - Round(X / DomainsPerBlock * MapScale * PageFactor)));
            Printer.Canvas.LineTo(Round(342 * PageFactor), Round(MapStart * PageFactor + Round(1 / DomainsPerBlock / 2 * MapScale * PageFactor) + 1 * PageFactor - Round(X / DomainsPerBlock * MapScale * PageFactor)));
          end;
      end;

      { Write right y-axis' label }
      Printer.Canvas.Font.Style := [fsBold];
      Printer.Canvas.TextOut(Round(355 * PageFactor), Round(25 * PageFactor), 'Pos.');
      Printer.Canvas.TextOut(Round(355 * PageFactor), Round(40 * PageFactor), '(kbp)');
      Printer.Canvas.Font.Style := [];
    end; { if }

  { Print melting curve }
  if FMain.MIViewMeltingCurve.Checked
    then
    begin
      { Draw melting curve in Printer }
      { Total melting curve }
      Printer.Canvas.Pen.Color := clBlack;
      X := 1;
      Printer.Canvas.MoveTo(Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale * PageFactor + 60 * PageFactor),
       Round(340 * PageFactor - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing * PageFactor)));

      for X := 2 to (MltDataPointNumber - 1) do
      begin
        { Runs off left side }
        if Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60) < 60
          then
            Printer.Canvas.MoveTo(Round(60 * PageFactor),
            Round(340 * PageFactor - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing * PageFactor)));

        { Runs off top }
        if ((340 - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing)) < 60)
         and not(Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60) < 60)
         and not(Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60) > 340)
          then
            Printer.Canvas.LineTo(Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale * PageFactor + 60 * PageFactor),
            Round(60 * PageFactor));

        { Runs off right side }
        if (Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60) > 340)
         and not((340 - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing)) < 60)
          then
            Printer.Canvas.LineTo(Round(340 * PageFactor),
            Round(340 * PageFactor - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing * PageFactor)));

        { Doesn't run off }
        if (Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60) >= 60)
         and ((340 - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing)) >= 60)
         and (Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60) <= 340)
          then
            Printer.Canvas.LineTo(Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale * PageFactor + 60 * PageFactor),
            Round(340 * PageFactor - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing * PageFactor)));
      end; { for }

      { Print MltTm }
      if FMain.MIViewMeltingCurveTm.Checked
        then
        begin
          Printer.Canvas.MoveTo(Round((MltTm - TempStart) * TempScale * PageFactor + 60 * PageFactor), Round(70 * PageFactor));
          Printer.Canvas.LineTo(Round((MltTm - TempStart) * TempScale * PageFactor + 60 * PageFactor), Round(85 * PageFactor));
          Str(MltTm:0:1, MltTmString);
          Printer.Canvas.TextOut(Round((MltTm - TempStart) * TempScale * PageFactor + 65 * PageFactor), Round(72 * PageFactor), MltTmString + 'C');
        end;

      PrintCommonParts;
  end; { if }

  { Print reference curve }
  if FMain.MIViewReferenceCurve.Checked
    then
    begin
      { Draw reference curve in Printer }
      { Total reference curve }
      Printer.Canvas.Pen.Color := clGreen;
      Printer.Canvas.Pen.Width := Round(0.5 * PageFactor);

      X := 1;
      Printer.Canvas.MoveTo(Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale * PageFactor + 60 * PageFactor),
       Round(340 * PageFactor - Round(TotalRefDataPoint[X].DthetaDT * VertSpacing * PageFactor)));

      for X := 2 to (RefDataPointNumber - 1) do
      begin
        { Runs off left side }
        if Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale + 60) < 60
          then
            Printer.Canvas.MoveTo(Round(60 * PageFactor),
            Round(340 * PageFactor - Round(TotalRefDataPoint[X].DthetaDT * VertSpacing * PageFactor)));

        { Runs off top }
        if ((340 - Round(TotalRefDataPoint[X].DthetaDT * VertSpacing)) < 60)
         and not(Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale + 60) < 60)
         and not(Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale + 60) > 340)
          then
            Printer.Canvas.LineTo(Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale * PageFactor + 60 * PageFactor),
            Round(60 * PageFactor));

        { Runs off right side }
        if (Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale + 60) > 340)
         and not((340 - Round(TotalRefDataPoint[X].DthetaDT * VertSpacing)) < 60)
          then
            Printer.Canvas.LineTo(Round(340 * PageFactor),
            Round(340 * PageFactor - Round(TotalRefDataPoint[X].DthetaDT * VertSpacing * PageFactor)));

        { Doesn't run off }
        if (Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale + 60) >= 60)
         and ((340 - Round(TotalRefDataPoint[X].DthetaDT * VertSpacing)) >= 60)
         and (Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale + 60) <= 340)
          then
            Printer.Canvas.LineTo(Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale * PageFactor + 60 * PageFactor),
            Round(340 * PageFactor - Round(TotalRefDataPoint[X].DthetaDT * VertSpacing * PageFactor)));
      end; { for }

      { Print RefTm }
      if FMain.MIViewReferenceCurveTm.Checked
        then
        begin
          Printer.Canvas.MoveTo(Round((RefTm - TempStart) * TempScale * PageFactor + 60 * PageFactor), Round(90 * PageFactor));
          Printer.Canvas.LineTo(Round((RefTm - TempStart) * TempScale * PageFactor + 60 * PageFactor), Round(105 * PageFactor));
          Str(RefTm:0:1, RefTmString);
          Printer.Canvas.TextOut(Round((RefTm - TempStart) * TempScale * PageFactor + 65 * PageFactor), Round(92 * PageFactor), RefTmString + 'C');
        end;

      Printer.Canvas.Pen.Width := Round(1 * PageFactor);

      PrintCommonParts;
    end; { if }

  Printer.EndDoc;
end;


(******************************************************************************)


end.
