unit URepaint;

interface

uses
  Classes, Graphics, SysUtils;

procedure RepaintPB1(Sender: TObject);

implementation

uses
  UMain, UAxes;


(******************************************************************************)


procedure RepaintPB1(Sender: TObject);

const
  XaxisStart           = 60;
  YaxisStart           = 60;
  XaxisFinal           = 340;
  YaxisFinal           = 340;
  XaxisSize            = 280;
  YaxisSize            = 280;

var
  MapMoveToX,
  MapMoveToY,
  MapLineToX,
  MapLineToY,
  MapStart,
  MapScale,
  X                    : Integer;
  TempStart,
  TempFinal,
  TempScale,
  LastTempUsed         : Real;
  MapString            : String;


  (**************************************************************************)


  procedure PaintCommonParts;
  { These parts are present on most graphs }

  var
    TickDegs,
    X         : Integer;

  begin { PaintCommonParts }
    { Refresh axes in PaintBox1 }
    { Draw axes }
    FMain.PaintBox1.Canvas.Pen.Color := clBlack;
    FMain.PaintBox1.Canvas.MoveTo(60, 60);
    FMain.PaintBox1.Canvas.LineTo(340, 60);
    FMain.PaintBox1.Canvas.LineTo(340, 340);
    FMain.PaintBox1.Canvas.LineTo(60, 340);
    FMain.PaintBox1.Canvas.LineTo(60, 60);

    { Draw x-axis, top and bottom }
    { Resolution at 0.01C }
    { Variable ticks }
    case (Round(TempFinal * 100) - Round(TempStart * 100)) of
      0..1000     : TickDegs := 100; { 0 to 10.00C gives 1.00C ticks }
      1001..4000  : TickDegs := 500; { 10.01C to 40.00C gives 5.00C ticks }
      else          TickDegs := 1000; { More than 40.00C gives 10.00C ticks }
    end; { case }
    for X := Round(TempStart * 100) to Round(TempFinal * 100) do
      if (X MOD TickDegs = 0) { Tick every TickDegs / 100 degrees }
       and (Round((X / 100 - TempStart) * TempScale + 60) >= 60)
       and (Round((X / 100 - TempStart) * TempScale + 60) <= 340)
        then
        begin
          { Bottom }
          FMain.PaintBox1.Canvas.MoveTo(Round((X / 100 - TempStart) * TempScale + 60), 340);
          FMain.PaintBox1.Canvas.LineTo(Round((X / 100 - TempStart) * TempScale + 60), 342);
          if X MOD (TickDegs * 2) = 0 { Major Tick every 2 minor }
            then
            begin
              FMain.PaintBox1.Canvas.LineTo(Round((X / 100 - TempStart) * TempScale + 60), 345);
              { Value }
              FMain.PaintBox1.Canvas.TextOut(Round((X / 100 - TempStart) * TempScale + 60 - 5), 351, IntToStr(Round(X / 100)));
            end;
          { Top }
          FMain.PaintBox1.Canvas.MoveTo(Round((X / 100 - TempStart) * TempScale + 60), 60);
          FMain.PaintBox1.Canvas.LineTo(Round((X / 100 - TempStart) * TempScale + 60), 58);
          if X MOD (TickDegs * 2) = 0 { Major Tick every 2 minor }
            then
              FMain.PaintBox1.Canvas.LineTo(Round((X / 100 - TempStart) * TempScale + 60), 55);
        end;
    { Label }
    FMain.PaintBox1.Canvas.Font.Style := [fsBold];
    FMain.PaintBox1.Canvas.TextOut(60, 10, TitleLine1);
    FMain.PaintBox1.Canvas.TextOut(60, 22, TitleLine2);
    FMain.PaintBox1.Canvas.TextOut(60, 34, TitleLine3);
    FMain.PaintBox1.Canvas.TextOut(154, 370, 'Temperature (C)');
    FMain.PaintBox1.Canvas.Font.Style := [];

    { Draw left y-axis }
    { Ticks }
    FMain.PaintBox1.Canvas.MoveTo(60, 60);
    FMain.PaintBox1.Canvas.LineTo(55, 60);
    FMain.PaintBox1.Canvas.MoveTo(60, 200);
    FMain.PaintBox1.Canvas.LineTo(55, 200);
    FMain.PaintBox1.Canvas.MoveTo(60, 340);
    FMain.PaintBox1.Canvas.LineTo(55, 340);
    { Values }
    FMain.PaintBox1.Canvas.TextOut(41, 334, '0');
    MltMax := 560 / VertSpacing * 0.5; { For y-axis max value }
    Str(MltMax:0:1, MltMaxString);
    FMain.PaintBox1.Canvas.TextOut(32, 54, MltMaxString);
    { Label }
    FMain.PaintBox1.Canvas.Font.Style := [fsBold];
    FMain.PaintBox1.Canvas.TextOut(8, 194, 'd');
    FMain.PaintBox1.Canvas.Font.Name := 'Symbol';
    FMain.PaintBox1.Canvas.TextOut(14, 194, 'q');
    FMain.PaintBox1.Canvas.Font.Name := 'Arial';
    FMain.PaintBox1.Canvas.TextOut(20, 194, '/dT');
    FMain.PaintBox1.Canvas.Font.Style := [];

    { Draw right y-axis ticks...non-positional plot }
    if not(FMain.MIViewMeltingMap.Checked)
      then
      begin
        FMain.PaintBox1.Canvas.MoveTo(340, 60);
        FMain.PaintBox1.Canvas.LineTo(345, 60);
        FMain.PaintBox1.Canvas.MoveTo(340, 200);
        FMain.PaintBox1.Canvas.LineTo(345, 200);
        FMain.PaintBox1.Canvas.MoveTo(340, 340);
        FMain.PaintBox1.Canvas.LineTo(345, 340);
      end;
  end; { PaintCommonParts }


  (**************************************************************************)


begin { RepaintPB1 }
  TempStart := StrToFloat(FAxes.ETempMin.Text);
  TempFinal := StrToFloat(FAxes.ETempMax.Text);
  TempScale := (XaxisFinal - XaxisStart) / (TempFinal - TempStart);

  VertSpacing := Round(560 * FAxes.UDMltMax.Position / 100); { default scale = 0.5 }

  FMain.PaintBox1.Canvas.Font.Name := 'Arial';
  FMain.PaintBox1.Canvas.Font.Size := 8;
  FMain.PaintBox1.Canvas.Pen.Width := 1;
  FMain.PaintBox1.Canvas.Font.Style := [];

  { Erase entire graph and redraw }
  if ClearPlots or (Sender = FAxes.BRedraw)
    then
    begin
      FMain.PaintBox1.Canvas.Brush.Color := clWhite;
      FMain.PaintBox1.Canvas.FillRect(Rect(0,0,400,400));
      ClearPlots := False;
    end;

  { Paint common parts in case nothing is selected }
  PaintCommonParts;

  { Paint melting map }
  { Drawn first to avoid erasing other curves }
  if FMain.MIViewMeltingMap.Checked
    then
    begin
      { Determine scale and bottom of positional plot }
      MapScale := Round(YaxisSize * DomainsPerBlock / (StrToInt(FAxes.ELastDomain.Text) - StrToInt(FAxes.EFirstDomain.Text) + 1));
      MapStart := 339 + Round((StrToInt(FAxes.EFirstDomain.Text) - 1) / DomainsPerBlock * MapScale);
      LastTempUsed := MapDataPoint[DomainsPerBlock + 1].Temp;

      for X := (DomainsPerBlock + 1) to MapDataPointNumber do
      begin
         { Determine temperature increment displayed }
        if (MapDataPoint[X].Temp >= LastTempUsed + StrToFloat(FAxes.ETempInc.Text))
         { Must be within boundaries }
         and (Round((MapDataPoint[X].Temp - TempStart) * TempScale + 60) > 60)
         and (Round((MapDataPoint[X].Temp - TempStart) * TempScale + 60) < 340)
         { Determine domains to be displayed }
         and (X >= Trunc((X - 1) / DomainsPerBlock) * DomainsPerBlock + StrToInt(FAxes.EFirstDomain.Text))
         and (X <= Trunc((X - 1) / DomainsPerBlock) * DomainsPerBlock + StrToInt(FAxes.ELastDomain.Text))
          then
          begin
            { Make LastTempUsed the current temp if next pass increases temp }
            if X = Trunc((X - 1) / DomainsPerBlock) * DomainsPerBlock + StrToInt(FAxes.ELastDomain.Text)
              then
                LastTempUsed := MapDataPoint[X].Temp;

            { Determine color to use }
            if MapDataPoint[X].Theta < 0.35
              then
                FMain.PaintBox1.Canvas.Pen.Color := $00e0e0e0; { Light gray }
            if (MapDataPoint[X].Theta >= 0.35) and (MapDataPoint[X].Theta < 0.45)
              then
                FMain.PaintBox1.Canvas.Pen.Color := clSilver;  { Medium gray }
            if (MapDataPoint[X].Theta >= 0.45) and (MapDataPoint[X].Theta < 0.55)
              then
                FMain.PaintBox1.Canvas.Pen.Color := $00404040; { Dark gray }
            if (MapDataPoint[X].Theta >= 0.55) and (MapDataPoint[X].Theta < 0.65)
              then
                FMain.PaintBox1.Canvas.Pen.Color := clSilver;  { Medium gray }
            if MapDataPoint[X].Theta >= 0.65
              then
                FMain.PaintBox1.Canvas.Pen.Color := clNone;

            { Draw melting map on PaintBox1 }
            MapMoveToX := Round((MapDataPoint[X].Temp - TempStart) * TempScale + 60);
            MapMoveToY := MapStart - Round((MapDataPoint[X].Pos - DOMAIN / 2)
             / (DOMAIN * DomainsPerBlock) * (MapScale - 1));
            MapLineToX := MapMoveToX;
            MapLineToY := MapStart - Round((MapDataPoint[X].Pos + DOMAIN / 2)
             / (DOMAIN * DomainsPerBlock) * (MapScale - 1)) + 1;

            FMain.PaintBox1.Canvas.MoveTo(MapMoveToX, MapMoveToY);
            FMain.PaintBox1.Canvas.LineTo(MapLineToX, MapLineToY);
          end; { if }
      end; { for }

      PaintCommonParts;

      { Remove left y-axis ticks, etc....may be redrawn later }
      FMain.PaintBox1.Canvas.Brush.Color := clWhite;
      FMain.PaintBox1.Canvas.FillRect(Rect(0,50,59,350));
      { Remove right y-axis ticks...to be replaced below }
      FMain.PaintBox1.Canvas.FillRect(Rect(341,50,400,350));

      { Draw right y-axis ticks, etc. }
      for X := StrToInt(FAxes.EFirstDomain.Text) to StrToInt(FAxes.ELastDomain.Text) do
      begin
        if X MOD 3 = 0 { Major tick every 3 domains }
          then
          begin
            FMain.PaintBox1.Canvas.MoveTo(340, (MapStart + Round(1 / DomainsPerBlock / 2 * MapScale) + 1 - Round(X / DomainsPerBlock * MapScale)));
            FMain.PaintBox1.Canvas.LineTo(345, (MapStart + Round(1 / DomainsPerBlock / 2 * MapScale) + 1 - Round(X / DomainsPerBlock * MapScale)));
            Str(((MapDataPoint[X].Pos + StrToInt(FAxes.ETicksLabeledFrom.Text) - 1) / 1000):0:StrToInt(FAxes.ELabelDecimalPlaces.Text), MapString);
            FMain.PaintBox1.Canvas.TextOut(355, (MapStart + Round(1 / DomainsPerBlock / 2 * MapScale) + 1 - Round(X / DomainsPerBlock * MapScale) - 6), MapString);
          end
          else { Minor ticks }
          begin
            FMain.PaintBox1.Canvas.MoveTo(340, (MapStart + Round(1 / DomainsPerBlock / 2 * MapScale) + 1 - Round(X / DomainsPerBlock * MapScale)));
            FMain.PaintBox1.Canvas.LineTo(342, (MapStart + Round(1 / DomainsPerBlock / 2 * MapScale) + 1 - Round(X / DomainsPerBlock * MapScale)));
          end;
      end;

      { Write right y-axis' label }
      FMain.PaintBox1.Canvas.Font.Style := [fsBold];
      FMain.PaintBox1.Canvas.TextOut(355, 25, 'Pos.');
      FMain.PaintBox1.Canvas.TextOut(355, 40, '(kbp)');
      FMain.PaintBox1.Canvas.Font.Style := [];
    end; { if }

  { Paint melting curve }
  if FMain.MIViewMeltingCurve.Checked
    then
    begin
      { Block melting curve }
      { Drawn only when melting in blocks }
      if BlockMltActive
        then
        begin
          { From highest to lowest }
          FMain.PaintBox1.Canvas.Pen.Color := clSilver;
          X := LastMltDataPoint;
          FMain.PaintBox1.Canvas.MoveTo(Round((BlockMltDataPoint[X].Temp - TempStart) * TempScale + 60),
           (340 - Round(BlockMltDataPoint[X].DthetaDT * VertSpacing)));

          if LastMltDataPoint > (MltDataPointNumber + 1)
            then
              for X := (LastMltDataPoint - 1) downto (MltDataPointNumber + 1) do
                { Doesn't run off top }
                if (340 - Round(BlockMltDataPoint[X].DthetaDT * VertSpacing)) > 60
                  then
                    FMain.PaintBox1.Canvas.LineTo(Round((BlockMltDataPoint[X].Temp - TempStart) * TempScale + 60),
                     (340 - Round(BlockMltDataPoint[X].DthetaDT * VertSpacing)))
                  else
                    FMain.PaintBox1.Canvas.LineTo(Round((BlockMltDataPoint[X].Temp - TempStart) * TempScale + 60),
                     60);

          { From lowest to highest }
          FMain.PaintBox1.Canvas.Pen.Color := clSilver;
          X := 1;
          FMain.PaintBox1.Canvas.MoveTo(Round((BlockMltDataPoint[X].Temp - TempStart) * TempScale + 60),
           (340 - Round(BlockMltDataPoint[X].DthetaDT * VertSpacing)));

          for X := 2 to (MltDataPointNumber - 1) do
            { Doesn't run off top }
            if (340 - Round(BlockMltDataPoint[X].DthetaDT * VertSpacing)) > 60
              then
                FMain.PaintBox1.Canvas.LineTo(Round((BlockMltDataPoint[X].Temp - TempStart) * TempScale + 60),
                 (340 - Round(BlockMltDataPoint[X].DthetaDT * VertSpacing)))
              else
                FMain.PaintBox1.Canvas.LineTo(Round((BlockMltDataPoint[X].Temp - TempStart) * TempScale + 60),
                 60);
        end; { if }

      { Total melting curve }
      { From highest to lowest }
      { Drawn only when melting in blocks }
      FMain.PaintBox1.Canvas.Pen.Color := clBlack;
      X := LastMltDataPoint;
      FMain.PaintBox1.Canvas.MoveTo(Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60),
       (340 - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing)));

      if LastMltDataPoint > (MltDataPointNumber + 1)
        then
          for X := (LastMltDataPoint - 1) downto (MltDataPointNumber + 1) do
            { Doesn't run off top }
            if (340 - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing)) > 60
              then
                FMain.PaintBox1.Canvas.LineTo(Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60),
                 (340 - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing)))
              else
                FMain.PaintBox1.Canvas.LineTo(Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60),
                 60);

      { From lowest to highest }
      { The only one drawn when not melting }
      FMain.PaintBox1.Canvas.Pen.Color := clBlack;
      X := 1;
      FMain.PaintBox1.Canvas.MoveTo(Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60),
       (340 - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing)));

      for X := 2 to (MltDataPointNumber - 1) do
      begin
        { Runs off left side }
        if Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60) < 60
          then
            FMain.PaintBox1.Canvas.MoveTo(60, (340 - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing)));

        { Runs off top }
        if ((340 - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing)) < 60)
         and not(Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60) < 60)
         and not(Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60) > 340)
          then
            FMain.PaintBox1.Canvas.LineTo(Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60), 60);

        { Runs off right side }
        if (Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60) > 340)
         and not((340 - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing)) < 60)
          then
            FMain.PaintBox1.Canvas.LineTo(340, (340 - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing)));

        { Doesn't run off }
        if (Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60) >= 60)
         and ((340 - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing)) >= 60)
         and (Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60) <= 340)
          then
            FMain.PaintBox1.Canvas.LineTo(Round((TotalMltDataPoint[X].Temp - TempStart) * TempScale + 60),
            Round(340 - Round(TotalMltDataPoint[X].DthetaDT * VertSpacing)));
      end; { for }

      { Paint MltTm }
      if FMain.MIViewMeltingCurveTm.Checked
        then
        begin
          FMain.PaintBox1.Canvas.MoveTo(Round((MltTm - TempStart) * TempScale + 60), 70);
          FMain.PaintBox1.Canvas.LineTo(Round((MltTm - TempStart) * TempScale + 60), 85);
          Str(MltTm:0:1, MltTmString);
          FMain.PaintBox1.Canvas.TextOut(Round((MltTm - TempStart) * TempScale + 65), 72, MltTmString + 'C');
        end;

      PaintCommonParts;
  end; { if }

  { Paint reference curve }
  if FMain.MIViewReferenceCurve.Checked
    then
    begin
      { From lowest to highest }
      FMain.PaintBox1.Canvas.Pen.Color := clGreen;
      X := 1;
      FMain.PaintBox1.Canvas.MoveTo(Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale + 60),
       (340 - Round(TotalRefDataPoint[X].DthetaDT * VertSpacing)));

      for X := 2 to (RefDataPointNumber - 1) do
      begin
        { Runs off left side }
        if Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale + 60) < 60
          then
            FMain.PaintBox1.Canvas.MoveTo(60, (340 - Round(TotalRefDataPoint[X].DthetaDT * VertSpacing)));

        { Runs off top }
        if ((340 - Round(TotalRefDataPoint[X].DthetaDT * VertSpacing)) < 60)
         and not(Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale + 60) < 60)
         and not(Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale + 60) > 340)
          then
            FMain.PaintBox1.Canvas.LineTo(Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale + 60), 60);

        { Runs off right side }
        if (Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale + 60) > 340)
         and not((340 - Round(TotalRefDataPoint[X].DthetaDT * VertSpacing)) < 60)
          then
            FMain.PaintBox1.Canvas.LineTo(340, (340 - Round(TotalRefDataPoint[X].DthetaDT * VertSpacing)));

        { Doesn't run off }
        if (Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale + 60) >= 60)
         and ((340 - Round(TotalRefDataPoint[X].DthetaDT * VertSpacing)) >= 60)
         and (Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale + 60) <= 340)
          then
            FMain.PaintBox1.Canvas.LineTo(Round((TotalRefDataPoint[X].Temp - TempStart) * TempScale + 60),
            Round(340 - Round(TotalRefDataPoint[X].DthetaDT * VertSpacing)));
      end; { for }

      { Paint RefTm }
      if FMain.MIViewReferenceCurveTm.Checked
        then
        begin
          FMain.PaintBox1.Canvas.MoveTo(Round((RefTm - TempStart) * TempScale + 60), 90);
          FMain.PaintBox1.Canvas.LineTo(Round((RefTm - TempStart) * TempScale + 60), 105);
          Str(RefTm:0:1, RefTmString);
          FMain.PaintBox1.Canvas.TextOut(Round((RefTm - TempStart) * TempScale + 65), 92, RefTmString + 'C');
        end;

      PaintCommonParts;
    end; { if }
end; { RepaintPB1 }


(******************************************************************************)


end.
