unit UMain;

{ MeltSim/Melt, Version 1.0 }
{ (C) Copyright, 1998, R.D. Blake & J.W. Bizzaro }

{ Min stack $00100000 }
{ Max stack $01E84800 }
{ Image base $00400000 }

interface

uses
  Windows, SysUtils, WinTypes, WinProcs, Messages, Classes, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, Buttons, ComCtrls, FMXUtils,
  OleCtrls, Gauges, Menus;

const
  ProgName = 'MeltSim/Melt 1.0';

type
  TFMain = class(TForm)
    OpenDialog1: TOpenDialog;
    MainMenu: TMainMenu;
    MIFileMenu: TMenuItem;
    MIMeltMenu: TMenuItem;
    MIMeltSimMenu: TMenuItem;
    MIHelpMenu: TMenuItem;
    MIOpenSequence: TMenuItem;
    MICloseSequence: TMenuItem;
    MIFileNames: TMenuItem;
    MIExitMelt: TMenuItem;
    MIBeginMelt: TMenuItem;
    MIEndMelt: TMenuItem;
    MIGraph: TMenuItem;
    MIHelpTopics: TMenuItem;
    N2: TMenuItem;
    N1: TMenuItem;
    Divider1: TBevel;
    StatusBar: TStatusBar;
    ProgressBar: TProgressBar;
    PResize: TPanel;
    IUpDown: TImage;
    GBMarkers: TGroupBox;
    CBMeltByMarkedBlocks: TCheckBox;
    GBMeltingBlocks: TGroupBox;
    LDomainsPerBlock: TLabel;
    LBlockSize: TLabel;
    Lbp: TLabel;
    EDomainsPerBlock: TEdit;
    EBlockSize: TEdit;
    GBTemperature: TGroupBox;
    LDegC1: TLabel;
    LStartingTemp: TLabel;
    LDegC2: TLabel;
    LDegC3: TLabel;
    LFinalTemp: TLabel;
    LTempInc: TLabel;
    ETempInc: TEdit;
    EStartingTemp: TEdit;
    EFinalTemp: TEdit;
    GBSalt: TGroupBox;
    LSodiumConc: TLabel;
    LM: TLabel;
    ESodiumConc: TEdit;
    GBSequences: TGroupBox;
    LStartAtBase: TLabel;
    LEndAtBase: TLabel;
    EStartAtBase: TEdit;
    EEndAtBase: TEdit;
    MICompile: TMenuItem;
    Memo: TMemo;
    LLog: TLabel;
    GBTitle: TGroupBox;
    ETitle: TEdit;
    MIMelt: TMenuItem;
    procedure ShowForm(Sender: TObject);
    procedure OpenSequence(Sender: TObject);
    procedure SetFileNames(Sender: TObject);
    procedure MICloseSequenceClick(Sender: TObject);
    procedure MIFileNamesClick(Sender: TObject);
    procedure MIExitMeltClick(Sender: TObject);
    procedure MIBeginMeltClick(Sender: TObject);
    procedure MIGraphClick(Sender: TObject);
    procedure MIHelpTopicsClick(Sender: TObject);
    procedure ResizeForm(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    procedure ThreadDone(Sender: TObject);
  public
    { Public declarations }
  end;

var
  FMain : TFMain;

implementation

uses
  UMelt, UFiles;

{$R *.DFM}

(******************************************************************************)
{ TFMain }

procedure TFMain.ShowForm(Sender: TObject);
begin

  if ParamCount > 0 then
  begin
    FFiles.ESeqFile.Text := ParamStr(1);
    OpenSequence(Sender);
  end;

end;


(******************************************************************************)


procedure TFMain.OpenSequence(Sender: TObject);
{ File is not kept opened, but the file name is determined }

var
  FirstChar  : Char;
  Title      : String;
  InFile     : TextFile;

begin

  if Sender = MIOpenSequence then
    if OpenDialog1.Execute then
      FFiles.ESeqFile.Text := OpenDialog1.FileName;

  { Open InFile }
  AssignFile(InFile, FFiles.ESeqFile.Text);
  Reset(InFile);

  { Extract title from InFile }
  Read(InFile, FirstChar);
  if FirstChar = '#'
    then
      Readln(InFile, Title) { Read title }
    else
      { Default title is InFile name }
      Title := ExtractFileName(FFiles.ESeqFile.Text);

  { Close InFile }
  CloseFile(InFile);

  { Set output file title and form caption }
  ETitle.Text := Title;
  Caption := ProgName + ' - ' + ExtractFileName(FFiles.ESeqFile.Text);

  { Set file names }
  SetFileNames(Sender);
end;


(******************************************************************************)


procedure TFMain.SetFileNames(Sender: TObject);
begin
  if ((FFiles.EMltFile.Text <> '.mlt') or (FFiles.EMapFile.Text <> '.pos')
   or (FFiles.EPro.Text <> '.pro') or (FFiles.ELogFile.Text <> '.log'))
   and (FFiles.ESeqFile.Text <> '.seq')
    then
    begin
      MessageBeep(MB_ICONQUESTION);
      if MessageDlg('Change output file names?', mtConfirmation,
       [mbYes, mbNo], 0) = mrYes
        then
        begin
          FFiles.EMltFile.Text := ChangeFileExt(FFiles.ESeqFile.Text, '.mlt');
          FFiles.EMapFile.Text := ChangeFileExt(FFiles.ESeqFile.Text, '.pos');
          FFiles.EPro.Text := ChangeFileExt(FFiles.ESeqFile.Text, '.pro');
          FFiles.ELogFile.Text := ChangeFileExt(FFiles.ESeqFile.Text, '.log');
        end;
    end;

  if (FFiles.EMltFile.Text = '.mlt') and (FFiles.EMapFile.Text = '.pos')
   and (FFiles.EPro.Text = '.pro') and (FFiles.ELogFile.Text = '.log')
   and (FFiles.ESeqFile.Text <> '.seq')
    then
    begin
      FFiles.EMltFile.Text := ChangeFileExt(FFiles.ESeqFile.Text, '.mlt');
      FFiles.EMapFile.Text := ChangeFileExt(FFiles.ESeqFile.Text, '.pos');
      FFiles.EPro.Text := ChangeFileExt(FFiles.ESeqFile.Text, '.pro');
      FFiles.ELogFile.Text := ChangeFileExt(FFiles.ESeqFile.Text, '.log');
    end;

end;


(******************************************************************************)


procedure TFMain.MICloseSequenceClick(Sender: TObject);
begin

{ put something here }

end;


(******************************************************************************)


procedure TFMain.MIExitMeltClick(Sender: TObject);

var
  Action : TCloseAction;

begin

  FormClose(Sender, Action);

end;


(******************************************************************************)


procedure TFMain.MIBeginMeltClick(Sender: TObject);

var
  I           : Integer;
  MsgReturned : String;

  (**************************************************************************)


  function InputDeterminedValid: Boolean;

  begin

    MsgReturned := '';

    { Check input - list most important here last }
    if not FileExists(FFiles.ESeqFile.Text)
      then MsgReturned := 'You must open a sequence file.';

    if MsgReturned = ''
      then
        InputDeterminedValid := True
      else
        InputDeterminedValid := False;

  end;


  (**************************************************************************)


begin { TFMain.MIBeginMeltClick }

  if InputDeterminedValid
    then
    begin
      { Input determined to be valid }
      with TMeltThread.Create(False) do
        OnTerminate := ThreadDone;

      { Reset StatusBar and disable some controls}
      StatusBar.Panels[0].Text := 'Working...';
      ProgressBar.Position := 0;
      MIOpenSequence.Enabled := False;
      MIBeginMelt.Enabled := False;
    end
    else
    begin
      { Input determined to be invalid }
      MessageBeep(MB_ICONEXCLAMATION);
      MessageDlg(MsgReturned, mtWarning, [mbOK], 0);
    end;

end; { TFMain.MIBeginMeltClick }


(******************************************************************************)


procedure TFMain.ThreadDone(Sender: TObject);
{ code performed at end of main routine }
begin

  { Reset StatusBar and enable some controls}
  StatusBar.Panels[0].Text := 'Ready.';
  ProgressBar.Position := 0;
  MIOpenSequence.Enabled := True;
  MIBeginMelt.Enabled := True;

end;


(******************************************************************************)


procedure TFMain.MIFileNamesClick(Sender: TObject);
begin

  FFiles.Visible := True;

end;


(******************************************************************************)


procedure TFMain.MIGraphClick(Sender: TObject);
begin

{ Put something here }

end;


(******************************************************************************)


procedure TFMain.MIHelpTopicsClick(Sender: TObject);

begin { TFMain.HelpTopicsClick }

  Application.HelpFile := 'LOGHELP.HLP';
  Application.HelpCommand(HELP_FINDER, 0);

{ ExecuteFile('loghelp.hlp', '', '', SW_SHOW); }

end; { TFMain.HelpTopicsClick }


(******************************************************************************)


procedure TFMain.ResizeForm(Sender: TObject);
begin

  if FMain.Height = 473
    then
    begin
      FMain.Height := 67;
      ProgressBar.Top := 8;
      PResize.Top := 7;
    end
    else
    begin
      FMain.Height := 473;
      ProgressBar.Top := 414;
      PResize.Top := 413;
    end;

end;


(******************************************************************************)


procedure TFMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  MessageBeep(MB_ICONQUESTION);

  if MessageDlg('Are you sure you want to exit Melt?', mtConfirmation,
    [mbYes, mbNo], 0) = mrYes
    then
    begin
      Application.Terminate;
    end
    else
      Action := caNone;
end;


(******************************************************************************)


procedure TFMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);

var
  Action : TCloseAction;

begin
  FormClose(Sender, Action);

  if Action = caNone
    then
      CanClose := False;
end;


(******************************************************************************)


end.
